@@ -1,12 +1,12 @@
 use std::io::Write;
 
-use crate::data::{self, entry::Header, input};
+use crate::data::{entry::Header, input};
 
 impl input::Entry {
     /// Create a new input entry from a given data `obj` set to be placed at the given `pack_offset`.
     ///
     /// This method is useful when arbitrary base entries are created
-    pub fn from_data_obj(obj: &data::Object<'_>, pack_offset: u64) -> Result<Self, input::Error> {
+    pub fn from_data_obj(obj: &git_object::Data<'_>, pack_offset: u64) -> Result<Self, input::Error> {
         let header = to_header(obj.kind);
         let compressed = compress_data(obj)?;
         let compressed_size = compressed.len() as u64;
@@ -30,7 +30,7 @@ impl input::Entry {
 
     /// Update our CRC value by recalculating it from our header and compressed data.
     pub fn compute_crc32(&self) -> u32 {
-        let mut header_buf = [0u8; 32];
+        let mut header_buf = [0u8; 12 + git_hash::Kind::longest().len_in_bytes()];
         let header_len = self
             .header
             .write_to(self.decompressed_size, header_buf.as_mut())
@@ -50,7 +50,7 @@ fn to_header(kind: git_object::Kind) -> Header {
     }
 }
 
-fn compress_data(obj: &data::Object<'_>) -> Result<Vec<u8>, input::Error> {
+fn compress_data(obj: &git_object::Data<'_>) -> Result<Vec<u8>, input::Error> {
     let mut out = git_features::zlib::stream::deflate::Write::new(Vec::new());
     if let Err(err) = std::io::copy(&mut &*obj.data, &mut out) {
         match err.kind() {
