@@ -1,6 +1,8 @@
-use std::{cmp::Ordering, collections::VecDeque, io};
+use std::{cmp::Ordering, io};
+
+pub(crate) const LARGE_OFFSET_THRESHOLD: u64 = 0x7fff_ffff;
+pub(crate) const HIGH_BIT: u32 = 0x8000_0000;
 
-use byteorder::{BigEndian, WriteBytesExt};
 use git_features::{
     hash,
     progress::{self, Progress},
@@ -10,7 +12,7 @@ use crate::index::{util::Count, V2_SIGNATURE};
 
 pub(crate) fn write_to(
     out: impl io::Write,
-    entries_sorted_by_oid: VecDeque<crate::cache::delta::Item<crate::index::write::TreeEntry>>,
+    entries_sorted_by_oid: Vec<crate::cache::delta::Item<crate::index::write::TreeEntry>>,
     pack_hash: &git_hash::ObjectId,
     kind: crate::index::Version,
     mut progress: impl Progress,
@@ -32,45 +34,15 @@ pub(crate) fn write_to(
         hash::Write::new(out, kind.hash()),
     ));
     out.write_all(V2_SIGNATURE)?;
-    out.write_u32::<BigEndian>(kind as u32)?;
+    out.write_all(&(kind as u32).to_be_bytes())?;
 
-    const LARGE_OFFSET_THRESHOLD: u64 = 0x7fff_ffff;
-    const HIGH_BIT: u32 = 0x8000_0000;
-
-    let needs_64bit_offsets =
-        entries_sorted_by_oid.back().expect("at least one pack entry").offset > LARGE_OFFSET_THRESHOLD;
-    let mut fan_out = [0u32; 256];
     progress.init(Some(4), progress::steps());
     let start = std::time::Instant::now();
-    let _info = progress.add_child("generating fan-out table");
+    let _info = progress.add_child("writing fan-out table");
+    let fan_out = fanout(entries_sorted_by_oid.iter().map(|e| e.data.id.first_byte()));
 
-    {
-        let mut iter = entries_sorted_by_oid.iter().enumerate();
-        let mut idx_and_entry = iter.next();
-        let mut upper_bound = 0;
-        let entries_len = entries_sorted_by_oid.len() as u32;
-
-        for (offset_be, byte) in fan_out.iter_mut().zip(0u8..=255) {
-            *offset_be = match idx_and_entry.as_ref() {
-                Some((_idx, entry)) => match entry.data.id.as_slice()[0].cmp(&byte) {
-                    Ordering::Less => unreachable!("ids should be ordered, and we make sure to keep ahead with them"),
-                    Ordering::Greater => upper_bound,
-                    Ordering::Equal => {
-                        idx_and_entry = iter.find(|(_, entry)| entry.data.id.as_slice()[0] != byte);
-                        upper_bound = match idx_and_entry.as_ref() {
-                            Some((idx, _)) => *idx as u32,
-                            None => entries_len,
-                        };
-                        upper_bound
-                    }
-                },
-                None => entries_len,
-            };
-        }
-    }
-
-    for value in fan_out {
-        out.write_u32::<BigEndian>(value)?;
+    for value in fan_out.iter() {
+        out.write_all(&value.to_be_bytes())?;
     }
 
     progress.inc();
@@ -82,7 +54,7 @@ pub(crate) fn write_to(
     progress.inc();
     let _info = progress.add_child("writing crc32");
     for entry in &entries_sorted_by_oid {
-        out.write_u32::<BigEndian>(entry.data.crc32)?;
+        out.write_all(&entry.data.crc32.to_be_bytes())?;
     }
 
     progress.inc();
@@ -90,7 +62,7 @@ pub(crate) fn write_to(
     {
         let mut offsets64 = Vec::<u64>::new();
         for entry in &entries_sorted_by_oid {
-            out.write_u32::<BigEndian>(if needs_64bit_offsets && entry.offset > LARGE_OFFSET_THRESHOLD {
+            let offset: u32 = if entry.offset > LARGE_OFFSET_THRESHOLD {
                 assert!(
                     offsets64.len() < LARGE_OFFSET_THRESHOLD as usize,
                     "Encoding breakdown - way too many 64bit offsets"
@@ -99,12 +71,11 @@ pub(crate) fn write_to(
                 ((offsets64.len() - 1) as u32) | HIGH_BIT
             } else {
                 entry.offset as u32
-            })?;
+            };
+            out.write_all(&offset.to_be_bytes())?;
         }
-        if needs_64bit_offsets {
-            for value in offsets64 {
-                out.write_u64::<BigEndian>(value)?;
-            }
+        for value in offsets64 {
+            out.write_all(&value.to_be_bytes())?;
         }
     }
 
@@ -121,7 +92,40 @@ pub(crate) fn write_to(
         start,
         (bytes_written_without_trailer + 20) as usize,
         progress::bytes().expect("unit always set"),
+        progress::MessageLevel::Success,
     );
 
     Ok(index_hash)
 }
+
+pub(crate) fn fanout(iter: impl ExactSizeIterator<Item = u8>) -> [u32; 256] {
+    let mut fan_out = [0u32; 256];
+    let entries_len = iter.len() as u32;
+    let mut iter = iter.enumerate();
+    let mut idx_and_entry = iter.next();
+    let mut upper_bound = 0;
+
+    for (offset_be, byte) in fan_out.iter_mut().zip(0u8..=255) {
+        *offset_be = match idx_and_entry.as_ref() {
+            Some((_idx, first_byte)) => match first_byte.cmp(&byte) {
+                Ordering::Less => unreachable!("ids should be ordered, and we make sure to keep ahead with them"),
+                Ordering::Greater => upper_bound,
+                Ordering::Equal => {
+                    if byte == 255 {
+                        entries_len
+                    } else {
+                        idx_and_entry = iter.find(|(_, first_byte)| *first_byte != byte);
+                        upper_bound = idx_and_entry
+                            .as_ref()
+                            .map(|(idx, _)| *idx as u32)
+                            .unwrap_or(entries_len);
+                        upper_bound
+                    }
+                }
+            },
+            None => entries_len,
+        };
+    }
+
+    fan_out
+}
