@@ -3,9 +3,9 @@ use std::{convert::TryInto, io, sync::atomic::AtomicBool};
 pub use error::Error;
 use git_features::progress::{self, Progress};
 
-use crate::cache::delta::{traverse::Context, Tree};
+use crate::cache::delta::{traverse, Tree};
 
-mod encode;
+pub(crate) mod encode;
 mod error;
 
 pub(crate) struct TreeEntry {
@@ -13,15 +13,6 @@ pub(crate) struct TreeEntry {
     pub crc32: u32,
 }
 
-impl Default for TreeEntry {
-    fn default() -> Self {
-        TreeEntry {
-            id: git_hash::ObjectId::null_sha1(),
-            crc32: 0,
-        }
-    }
-}
-
 /// Information gathered while executing [`write_data_iter_to_stream()`][crate::index::File::write_data_iter_to_stream]
 #[derive(PartialEq, Eq, Debug, Hash, Ord, PartialOrd, Clone)]
 #[cfg_attr(feature = "serde1", derive(serde::Serialize, serde::Deserialize))]
@@ -47,6 +38,7 @@ impl crate::index::File {
     /// `tread_limit` is used for a parallel tree traversal for obtaining object hashes with optimal performance.
     /// `root_progress` is the top-level progress to stay informed about the progress of this potentially long-running
     /// computation.
+    /// `object_hash` defines what kind of object hash we write into the index file.
     ///
     /// # Remarks
     ///
@@ -55,6 +47,7 @@ impl crate::index::File {
     /// provides all bytes belonging to a pack entry writing them to the given mutable output `Vec`.
     /// It should return `None` if the entry cannot be resolved from the pack that produced the `entries` iterator, causing
     /// the write operation to fail.
+    #[allow(clippy::too_many_arguments)]
     pub fn write_data_iter_to_stream<F, F2>(
         kind: crate::index::Version,
         make_resolver: F,
@@ -63,6 +56,7 @@ impl crate::index::File {
         mut root_progress: impl Progress,
         out: impl io::Write,
         should_interrupt: &AtomicBool,
+        object_hash: git_hash::Kind,
     ) -> Result<Outcome, Error>
     where
         F: FnOnce() -> io::Result<F2>,
@@ -113,7 +107,7 @@ impl crate::index::File {
                     tree.add_root(
                         pack_offset,
                         TreeEntry {
-                            id: git_hash::ObjectId::null_sha1(),
+                            id: object_hash.null(),
                             crc32,
                         },
                     )?;
@@ -131,7 +125,7 @@ impl crate::index::File {
                         base_pack_offset,
                         pack_offset,
                         TreeEntry {
-                            id: git_hash::ObjectId::null_sha1(),
+                            id: object_hash.null(),
                             crc32,
                         },
                     )?;
@@ -162,18 +156,14 @@ impl crate::index::File {
         let resolver = make_resolver()?;
         let sorted_pack_offsets_by_oid = {
             let in_parallel_if_pack_is_big_enough = || bytes_to_process > 5_000_000;
-            let mut items = tree.traverse(
+            let traverse::Outcome { roots, children } = tree.traverse(
                 in_parallel_if_pack_is_big_enough,
                 resolver,
-                root_progress.add_child("Resolving"),
-                root_progress.add_child("Decoding"),
-                thread_limit,
-                should_interrupt,
                 pack_entries_end,
                 || (),
                 |data,
                  _progress,
-                 Context {
+                 traverse::Context {
                      entry,
                      decompressed: bytes,
                      ..
@@ -181,12 +171,21 @@ impl crate::index::File {
                     modify_base(data, entry, bytes, kind.hash());
                     Ok::<_, Error>(())
                 },
+                crate::cache::delta::traverse::Options {
+                    object_progress: root_progress.add_child("Resolving"),
+                    size_progress: root_progress.add_child("Decoding"),
+                    thread_limit,
+                    should_interrupt,
+                    object_hash,
+                },
             )?;
             root_progress.inc();
 
+            let mut items = roots;
+            items.extend(children);
             {
                 let _progress = root_progress.add_child("sorting by id");
-                items.make_contiguous().sort_by_key(|e| e.data.id);
+                items.sort_by_key(|e| e.data.id);
             }
 
             root_progress.inc();
@@ -205,6 +204,7 @@ impl crate::index::File {
             indexing_start,
             num_objects as usize,
             progress::count("objects").expect("unit always set"),
+            progress::MessageLevel::Success,
         );
         Ok(Outcome {
             index_kind: kind,
@@ -221,8 +221,8 @@ fn modify_base(
     decompressed: &[u8],
     hash: git_hash::Kind,
 ) {
-    fn compute_hash(kind: git_object::Kind, bytes: &[u8], hash_kind: git_hash::Kind) -> git_hash::ObjectId {
-        let mut hasher = git_features::hash::hasher(hash_kind);
+    fn compute_hash(kind: git_object::Kind, bytes: &[u8], object_hash: git_hash::Kind) -> git_hash::ObjectId {
+        let mut hasher = git_features::hash::hasher(object_hash);
         hasher.update(&git_object::encode::loose_header(kind, bytes.len()));
         hasher.update(bytes);
         git_hash::ObjectId::from(hasher.digest())
